"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const ontype_formatter_1 = require("./providers/ontype_formatter");
const range_formatter_1 = require("./providers/range_formatter");
const formatter = require("./format");
const inferer = require("./infer");
const docmirror = require("./docmirror");
const ClojureLanguageConfiguration = {
    wordPattern: /[^\s,#()[\]{};"\\]+/,
    onEnterRules: [
        // This is madness, but the only way to stop vscode from indenting new lines
        {
            beforeText: /.*/,
            action: {
                indentAction: vscode.IndentAction.Outdent,
                removeText: Number.MAX_VALUE
            }
        },
    ]
};
function activate(context) {
    docmirror.activate();
    vscode.languages.setLanguageConfiguration("clojure", ClojureLanguageConfiguration);
    // this doesn't actually grow anything yet, but just jumps to the start of the enclosing expression.
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.forwardSexp', docmirror.forwardSexp))
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.backwardSexp', docmirror.backwardSexp))
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.forwardList', docmirror.forwardList))
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.backwardList', docmirror.backwardList))
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.downList', docmirror.downList))
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.upList', docmirror.upList))
    // context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.backwardUpList', docmirror.backwardUpList))
    context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.formatCurrentForm', formatter.formatPositionCommand));
    context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.alignCurrentForm', formatter.alignPositionCommand));
    context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.inferParens', inferer.inferParensCommand));
    context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.tabIndent', (e) => { inferer.indentCommand(e, " ", true); }));
    context.subscriptions.push(vscode.commands.registerTextEditorCommand('calva-fmt.tabDedent', (e) => { inferer.indentCommand(e, " ", false); }));
    context.subscriptions.push(vscode.languages.registerOnTypeFormattingEditProvider("clojure", new ontype_formatter_1.FormatOnTypeEditProvider, "\r", "\n"));
    context.subscriptions.push(vscode.languages.registerDocumentRangeFormattingEditProvider("clojure", new range_formatter_1.RangeEditProvider));
    vscode.window.onDidChangeActiveTextEditor(inferer.updateState);
}
exports.activate = activate;
//# sourceMappingURL=extension.js.map