"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const config = require("./config");
const { formatTextAtRange, formatTextAtIdx, formatTextAtIdxOnType, cljify, jsify } = require('../../../cljs-out/calva-fmt/cljs-lib');
function formatRangeEdits(document, range) {
    const text = document.getText(range), rangeTuple = [document.offsetAt(range.start), document.offsetAt(range.end)], newText = _formatRange(text, document.getText(), rangeTuple, document.eol == 2 ? "\r\n" : "\n");
    return [vscode.TextEdit.replace(range, newText)];
}
exports.formatRangeEdits = formatRangeEdits;
function formatRange(document, range) {
    let wsEdit = new vscode.WorkspaceEdit();
    wsEdit.set(document.uri, formatRangeEdits(document, range));
    return vscode.workspace.applyEdit(wsEdit);
}
exports.formatRange = formatRange;
function formatPosition(editor, onType = false, extraConfig = {}) {
    const doc = editor.document, pos = editor.selection.active, index = doc.offsetAt(pos), formatted = _formatIndex(doc.getText(), index, doc.eol == 2 ? "\r\n" : "\n", onType, extraConfig), range = new vscode.Range(doc.positionAt(formatted.range[0]), doc.positionAt(formatted.range[1])), newIndex = doc.offsetAt(range.start) + formatted["new-index"], previousText = doc.getText(range);
    if (previousText != formatted["range-text"]) {
        editor.edit(textEditorEdit => {
            textEditorEdit.replace(range, formatted["range-text"]);
        }, { undoStopAfter: false, undoStopBefore: false }).then((_onFulfilled) => {
            editor.selection = new vscode.Selection(doc.positionAt(newIndex), doc.positionAt(newIndex));
        });
    }
    else {
        if (newIndex != index) {
            editor.selection = new vscode.Selection(doc.positionAt(newIndex), doc.positionAt(newIndex));
        }
    }
}
exports.formatPosition = formatPosition;
function formatPositionCommand(editor) {
    formatPosition(editor);
}
exports.formatPositionCommand = formatPositionCommand;
function alignPositionCommand(editor) {
    formatPosition(editor, true, { "align-associative?": true });
}
exports.alignPositionCommand = alignPositionCommand;
function _formatIndex(allText, index, eol, onType = false, extraConfig = {}) {
    const d = cljify({
        "all-text": allText,
        "idx": index,
        "eol": eol,
        "config": Object.assign({}, config.getConfig(), extraConfig)
    }), result = jsify(onType ? formatTextAtIdxOnType(d) : formatTextAtIdx(d));
    if (!result["error"]) {
        return result;
    }
    else {
        console.log(result["error"]);
        return null;
    }
}
function _formatRange(rangeText, allText, range, eol) {
    const d = {
        "range-text": rangeText,
        "all-text": allText,
        "range": range,
        "eol": eol,
        "config": config.getConfig()
    }, cljData = cljify(d), result = jsify(formatTextAtRange(cljData));
    if (!result["error"]) {
        return result["range-text"];
    }
    else {
        console.log(result["error"]);
        return rangeText;
    }
}
//# sourceMappingURL=format.js.map