"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const { inferParens, inferIndents } = require('../../../cljs-out/calva-fmt/cljs-lib');
function inferParensCommand(editor) {
    const position = editor.selection.active, document = editor.document, currentText = document.getText(), r = inferParens({
        "text": currentText,
        "line": position.line,
        "character": position.character,
        "previous-line": position.line,
        "previous-character": position.character
    });
    applyResults(r, editor);
}
exports.inferParensCommand = inferParensCommand;
function indentCommand(editor, spacing, forward = true) {
    const prevPosition = editor.selection.active, document = editor.document;
    let deletedText = "", doEdit = true;
    editor.edit(editBuilder => {
        if (forward) {
            editBuilder.insert(new vscode.Position(prevPosition.line, prevPosition.character), spacing);
        }
        else {
            const startOfLine = new vscode.Position(prevPosition.line, 0), headRange = new vscode.Range(startOfLine, prevPosition), headText = document.getText(headRange), xOfFirstLeadingSpace = headText.search(/ *$/), leadingSpaces = xOfFirstLeadingSpace >= 0 ? prevPosition.character - xOfFirstLeadingSpace : 0;
            if (leadingSpaces > 0) {
                const spacingSize = Math.max(spacing.length, 1), deleteRange = new vscode.Range(prevPosition.translate(0, -spacingSize), prevPosition);
                deletedText = document.getText(deleteRange);
                editBuilder.delete(deleteRange);
            }
            else {
                doEdit = false;
            }
        }
    }, { undoStopAfter: false, undoStopBefore: false }).then((_onFulfilled) => {
        if (doEdit) {
            const position = editor.selection.active, currentText = document.getText(), r = inferIndents({
                "text": currentText,
                "line": position.line,
                "character": position.character,
                "previous-line": prevPosition.line,
                "previous-character": prevPosition.character,
                "changes": [{
                        "line": forward ? prevPosition.line : position.line,
                        "character": forward ? prevPosition.character : position.character,
                        "old-text": forward ? "" : deletedText,
                        "new-text": forward ? spacing : ""
                    }]
            });
            applyResults(r, editor);
        }
    });
}
exports.indentCommand = indentCommand;
function applyResults(r, editor) {
    if (r.success) {
        editor.edit(editBuilder => {
            r.edits.forEach((edit) => {
                const start = new vscode.Position(edit.start.line, edit.start.character), end = new vscode.Position(edit.end.line, edit.end.character);
                editBuilder.replace(new vscode.Range(start, end), edit.text);
            });
        }, { undoStopAfter: true, undoStopBefore: false }).then((_onFulfilled) => {
            const newPosition = new vscode.Position(r.line, r.character);
            editor.selections = [new vscode.Selection(newPosition, newPosition)];
        });
    }
    else {
        vscode.window.showErrorMessage("Calva Formatter Error: " + (r.error ? r.error.message : r["error-msg"]));
    }
}
function updateState(editor) {
}
exports.updateState = updateState;
//# sourceMappingURL=infer.js.map