"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const fs = require("fs");
const state = require("./state");
const util = require("./utilities");
const shadow_1 = require("./shadow");
const status_1 = require("./status");
const terminal_1 = require("./terminal");
const nrepl_1 = require("./nrepl");
function nreplPortFile() {
    if (fs.existsSync(shadow_1.default.shadowNReplPortFile()))
        return shadow_1.default.shadowNReplPortFile();
    else
        return util.getProjectDir() + '/.nrepl-port';
}
function disconnect(options = null, callback = () => { }) {
    ['clj', 'cljs'].forEach(sessionType => {
        state.cursor.set(sessionType, null);
    });
    state.cursor.set("connected", false);
    state.cursor.set('cljc', null);
    status_1.default.update();
    exports.nClient.close();
    callback();
}
function connectToHost(hostname, port) {
    return __awaiter(this, void 0, void 0, function* () {
        let chan = state.deref().get('outputChannel');
        if (exports.nClient) {
            exports.nClient["silent"] = true;
            exports.nClient.close();
        }
        state.cursor.set('connecting', true);
        status_1.default.update();
        try {
            chan.appendLine("Hooking up nREPL sessions...");
            exports.nClient = yield nrepl_1.NReplClient.create({ host: hostname, port: +port });
            exports.nClient.onClose(c => {
                state.cursor.set("connected", false);
                state.cursor.set("connecting", false);
                if (!c["silent"])
                    chan.appendLine("nREPL Connection was closed");
                status_1.default.update();
            });
            exports.cljSession = exports.nClient.session;
            chan.appendLine("Connected session: clj");
            state.cursor.set("connected", true);
            state.cursor.set("connecting", false);
            state.cursor.set('clj', exports.cljSession);
            state.cursor.set('cljc', exports.cljSession);
            status_1.default.update();
            //cljsSession = nClient.session;
            terminal_1.default.createREPLTerminal('clj', null, chan);
            let [cljsSession, shadowBuild] = yield makeCljsSessionClone(exports.cljSession, null);
            if (cljsSession)
                setUpCljsRepl(cljsSession, chan, shadowBuild);
            chan.appendLine('cljc files will use the clj REPL.' + (cljsSession ? ' (You can toggle this at will.)' : ''));
            //evaluate.evaluateFile();
            status_1.default.update();
        }
        catch (e) {
            state.cursor.set("connected", false);
            state.cursor.set("connecting", false);
            chan.appendLine("Failed connecting. (Calva needs a REPL started before it can connect.)");
        }
        /*
        disconnect({ hostname, port }, () => {
            let onDisconnect = (_client, err) => {
                chan.appendLine("Disconnected from nREPL server." + (err ? " Error: " + err : ""));
                state.cursor.set("clj", null);
                state.cursor.set("cljc", null);
                state.cursor.set("connected", false);
                state.cursor.set("connecting", false);
                status.update();
            }
        });
        */
    });
}
function setUpCljsRepl(cljsSession, chan, shadowBuild) {
    state.cursor.set("cljs", cljsSession);
    chan.appendLine("Connected session: cljs");
    terminal_1.default.createREPLTerminal('cljs', shadowBuild, chan);
    status_1.default.update();
}
function makeCljsSessionClone(session, shadowBuild) {
    return __awaiter(this, void 0, void 0, function* () {
        let chan = state.deref().get('outputChannel');
        if (shadow_1.default.isShadowCljs() && !shadowBuild) {
            chan.appendLine("This looks like a shadow-cljs coding session.");
            let build = yield vscode.window.showQuickPick(shadow_1.default.shadowBuilds(), {
                placeHolder: "Select which shadow-cljs CLJS REPL to connect to",
                ignoreFocusOut: true
            });
            if (build)
                return makeCljsSessionClone(session, build);
        }
        else {
            exports.cljsSession = yield exports.cljSession.clone();
            if (exports.cljsSession) {
                let isFigwheel = !shadowBuild;
                let initCode = shadowBuild ? shadowCljsReplStart(shadowBuild) : util.getCljsReplStartCode();
                let err = [];
                let out = [];
                let result = exports.cljsSession.eval(initCode, { stderr: x => err.push(x), stdout: x => out.push(x) });
                try {
                    let valueResult = yield result.value;
                    if (!shadowBuild && result.ns) {
                        state.cursor.set('shadowBuild', null);
                        if (result.ns === exports.cljSession.client.ns && out.find(x => { return x.search("not initialized"); })) {
                            tellUserFigwheelNotStarted(chan);
                        }
                        else {
                            state.cursor.set('cljs', exports.cljsSession);
                            return [exports.cljsSession, null];
                        }
                    }
                    else if (shadowBuild && valueResult.match(/:selected/)) {
                        state.cursor.set('shadowBuild', shadowBuild);
                        return [exports.cljsSession, shadowBuild];
                    }
                }
                catch (e) {
                    if (shadowBuild) {
                        let failed = `Failed starting cljs repl for shadow-cljs build: ${shadowBuild}`;
                        state.cursor.set('shadowBuild', null);
                        chan.appendLine(`${failed}. Is the build running and conected?`);
                        console.error(failed);
                    }
                    else {
                        tellUserFigwheelNotStarted(chan);
                    }
                }
            }
        }
        return [null, null];
    });
}
function tellUserFigwheelNotStarted(chan) {
    chan.appendLine("Figwheel not started:");
    chan.appendLine("  If you want a cljs repl, start Figwheel from the cljs REPL using: ");
    chan.appendLine("    (require '[figwheel-sidecar.repl-api :as fw])(fw/start-figwheel!)(fw/cljs-repl)");
    chan.appendLine("  and connect to the app with the browser. Then reconnect Calva.");
}
function shadowCljsReplStart(buildOrRepl) {
    if (!buildOrRepl)
        return null;
    if (buildOrRepl.charAt(0) == ":")
        return `(shadow.cljs.devtools.api/nrepl-select ${buildOrRepl})`;
    else
        return `(shadow.cljs.devtools.api/${buildOrRepl})`;
}
function promptForNreplUrlAndConnect(port) {
    return __awaiter(this, void 0, void 0, function* () {
        let current = state.deref(), chan = current.get('outputChannel');
        let url = yield vscode.window.showInputBox({
            placeHolder: "Enter existing nREPL hostname:port here...",
            prompt: "Add port to nREPL if localhost, otherwise 'hostname:port'",
            value: "localhost:" + (port ? port : ""),
            ignoreFocusOut: true
        });
        // state.reset(); TODO see if this should be done
        if (url !== undefined) {
            let [hostname, port] = url.split(':'), parsedPort = parseFloat(port);
            if (parsedPort && parsedPort > 0 && parsedPort < 65536) {
                state.cursor.set("hostname", hostname);
                state.cursor.set("port", parsedPort);
                connectToHost(hostname, parsedPort);
            }
            else {
                chan.appendLine("Bad url: " + url);
                state.cursor.set('connecting', false);
                status_1.default.update();
            }
        }
        else {
            state.cursor.set('connecting', false);
            status_1.default.update();
        }
    });
}
function connect(isAutoConnect = false) {
    let current = state.deref(), chan = current.get('outputChannel');
    new Promise((resolve, reject) => {
        if (fs.existsSync(nreplPortFile())) {
            fs.readFile(nreplPortFile(), 'utf8', (err, data) => {
                if (!err) {
                    resolve(parseFloat(data));
                }
                else {
                    reject(err);
                }
            });
        }
        else {
            resolve(null);
        }
    }).then((port) => {
        if (port) {
            if (isAutoConnect) {
                state.cursor.set("hostname", "localhost");
                state.cursor.set("port", port);
                connectToHost("localhost", port);
            }
            else {
                promptForNreplUrlAndConnect(port);
            }
        }
        else {
            chan.appendLine('No nrepl port file found. (Calva does not start the nrepl for you, yet.) You might need to adjust "calva.projectRootDirectory" in Workspace Settings.');
            promptForNreplUrlAndConnect(port);
        }
    }).catch((err) => {
        chan.appendLine("Error reading nrepl port file: " + err);
        promptForNreplUrlAndConnect(null);
    });
}
function reconnect() {
    state.reset();
    connect(true);
}
function autoConnect() {
    connect(true);
}
function toggleCLJCSession() {
    let current = state.deref();
    if (current.get('connected')) {
        if (util.getSession('cljc') == util.getSession('cljs')) {
            state.cursor.set('cljc', util.getSession('clj'));
        }
        else if (util.getSession('cljc') == util.getSession('clj')) {
            state.cursor.set('cljc', util.getSession('cljs'));
        }
        status_1.default.update();
    }
}
function recreateCljsRepl() {
    return __awaiter(this, void 0, void 0, function* () {
        let current = state.deref(), cljSession = util.getSession('clj'), chan = current.get('outputChannel');
        let [session, shadowBuild] = yield makeCljsSessionClone(cljSession, null);
        if (session)
            setUpCljsRepl(session, chan, shadowBuild);
        status_1.default.update();
    });
}
exports.default = {
    connect,
    disconnect,
    reconnect,
    autoConnect,
    toggleCLJCSession,
    recreateCljsRepl
};
//# sourceMappingURL=connector.js.map