"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const state = require("./state");
const status_1 = require("./status");
const connector_1 = require("./connector");
const terminal_1 = require("./terminal");
const completion_1 = require("./providers/completion");
const content_1 = require("./providers/content");
const hover_1 = require("./providers/hover");
const definition_1 = require("./providers/definition");
const evaluate_1 = require("./repl/middleware/evaluate");
const lint_1 = require("./repl/middleware/lint");
const testRunner_1 = require("./repl/middleware/testRunner");
const annotations_1 = require("./providers/annotations");
const select_1 = require("./repl/middleware/select");
const evaluate_2 = require("./repl/middleware/evaluate");
const greetings = require('@cospaia/calva-lib/lib/calva.greet');
function onDidSave(document) {
    let { evaluate, lint, test } = state.config();
    if (document.languageId !== 'clojure') {
        return;
    }
    if (test) {
        if (test) {
            testRunner_1.default.runNamespaceTests(document);
        }
    }
    else if (evaluate) {
        evaluate_1.default.evaluateFile(document);
    }
    if (lint) {
        lint_1.default.lintDocument(document);
    }
}
function onDidOpen(document) {
    if (document.languageId !== 'clojure') {
        return;
    }
    if (state.config().lint) {
        lint_1.default.lintDocument(document);
    }
}
function activate(context) {
    let chan = state.deref().get('outputChannel');
    chan.appendLine("Calva activated.");
    let { autoConnect, lint, useWSL } = state.config();
    status_1.default.update();
    // COMMANDS
    /*
    context.subscriptions.push(vscode.commands.registerCommand('calva.openReplWindow', async function () {
        const panel = vscode.window.createWebviewPanel("replInteractor", "REPL Interactor", vscode.ViewColumn.Active, { retainContextWhenHidden: true, enableScripts: true, localResourceRoots: [vscode.Uri.file(path.join(context.extensionPath, 'html'))] })
        let html = readFileSync(path.join(context.extensionPath, "html/index.html")).toString()
        html = html.replace("{{baseUri}}", getUrl())
        html = html.replace("{{script}}", getUrl("main.js"))
        html = html.replace("{{logo}}", getUrl("/clojure-logo.svg"))
        panel.webview.html = html;
        
        let session = await nClient.createSession();

        let res = session.eval("*ns*");
        await res.value;
        let ns = res.ns;

        panel.webview.onDidReceiveMessage(async function (msg) {
            if(msg.type == "init") {
                panel.webview.postMessage({ type: "init", value: "", ns: ns });
            }

            if(msg.type == "read-line") {
                let res = session.eval(msg.line, {
                            stderr: m => panel.webview.postMessage({type: "stderr", value: m}),
                            stdout: m => panel.webview.postMessage({type: "stdout", value: m})})
                try {
                    panel.webview.postMessage({type: "repl-response", value: await res.value, ns: res.ns});
                } catch(e) {
                    panel.webview.postMessage({type: "repl-error", ex: e});
                }
            }
        })
    }));
    */
    context.subscriptions.push(vscode.commands.registerCommand('calva.connect', connector_1.default.connect));
    context.subscriptions.push(vscode.commands.registerCommand('calva.reconnect', connector_1.default.reconnect));
    context.subscriptions.push(vscode.commands.registerCommand('calva.toggleCLJCSession', connector_1.default.toggleCLJCSession));
    context.subscriptions.push(vscode.commands.registerCommand('calva.recreateCljsRepl', connector_1.default.recreateCljsRepl));
    context.subscriptions.push(vscode.commands.registerCommand('calva.selectCurrentForm', select_1.default.selectCurrentForm));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateFile', evaluate_1.default.evaluateFile));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateSelection', evaluate_1.default.evaluateSelection));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateCurrentTopLevelForm', evaluate_1.default.evaluateTopLevelForm));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateSelectionPrettyPrint', evaluate_1.default.evaluateSelectionPrettyPrint));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateCurrentTopLevelFormPrettyPrint', evaluate_1.default.evaluateCurrentTopLevelFormPrettyPrint));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateSelectionReplace', evaluate_1.default.evaluateSelectionReplace));
    context.subscriptions.push(vscode.commands.registerCommand('calva.lintFile', lint_1.default.lintDocument));
    context.subscriptions.push(vscode.commands.registerCommand('calva.runNamespaceTests', testRunner_1.default.runNamespaceTestsCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.runAllTests', testRunner_1.default.runAllTestsCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.rerunTests', testRunner_1.default.rerunTestsCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.openREPLTerminal', terminal_1.default.openREPLTerminalCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.loadNamespace', terminal_1.default.loadNamespaceCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.setREPLNamespace', terminal_1.default.setREPLNamespaceCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evalCurrentFormInREPLTerminal', terminal_1.default.evalCurrentFormInREPLTerminalCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evalCurrentTopLevelFormInREPLTerminal', terminal_1.default.evalCurrentTopLevelFormInREPLTerminalCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.clearInlineResults', annotations_1.default.clearEvaluationDecorations));
    context.subscriptions.push(vscode.commands.registerCommand('calva.copyLastResults', evaluate_2.default.copyLastResultCommand));
    // PROVIDERS
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider(state.mode, new completion_1.default()));
    context.subscriptions.push(vscode.languages.registerHoverProvider(state.mode, new hover_1.default()));
    context.subscriptions.push(vscode.languages.registerDefinitionProvider(state.mode, useWSL ? new definition_1.WslDefinitionProvider() : new definition_1.DefinitionProvider()));
    vscode.workspace.registerTextDocumentContentProvider('jar', new content_1.default());
    // //EVENTS
    context.subscriptions.push(vscode.workspace.onDidOpenTextDocument((document) => {
        onDidOpen(document);
    }));
    context.subscriptions.push(vscode.workspace.onDidSaveTextDocument((document) => {
        onDidSave(document);
    }));
    context.subscriptions.push(vscode.window.onDidChangeActiveTextEditor((editor) => {
        status_1.default.update();
        if (state.config().syncReplNamespaceToCurrentFile) {
            terminal_1.default.setREPLNamespace();
        }
    }));
    context.subscriptions.push(vscode.workspace.onDidChangeTextDocument(annotations_1.default.onDidChangeTextDocument));
    context.subscriptions.push(new vscode.Disposable(() => {
        connector_1.default.disconnect();
        chan.dispose();
    }));
    // context.subscriptions.push(vscode.workspace.onDidChangeConfiguration((event) => {
    //     console.log(event);
    // }));
    vscode.commands.executeCommand('setContext', 'calva:activated', true);
    greetings.activationGreetings(chan, lint);
    //Try to connect using an existing .nrepl-port file, searching the root-directory
    if (autoConnect) {
        chan.appendLine("Autoconnecting... (This can be disabled in Settings)");
        connector_1.default.autoConnect();
    }
    else {
        chan.appendLine("Autoconnect disabled in Settings.");
    }
    // REPL
    function getUrl(name) {
        if (name)
            return vscode.Uri.file(path.join(context.extensionPath, "html", name)).with({ scheme: 'vscode-resource' }).toString();
        else
            return vscode.Uri.file(path.join(context.extensionPath, "html")).with({ scheme: 'vscode-resource' }).toString();
    }
}
exports.activate = activate;
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map