'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const clipboardy = require("clipboardy");
function toCommand([command, start, arg]) {
    if (command === 'insert')
        return { kind: command, start: start, text: arg };
    else
        return { kind: command, start: start, length: arg };
}
function toVscodePositions(editor, pos) {
    let start, end;
    if (typeof pos === "number")
        start = end = pos;
    else if (pos instanceof Array)
        start = pos[0], end = pos[1];
    let pos1 = editor.document.positionAt(start), pos2 = editor.document.positionAt(end);
    return { pos1, pos2 };
}
exports.commands = (res) => res.changes.map(toCommand);
function end(command) {
    if (command.kind === 'insert')
        return command.start + command.text.length;
    else
        return command.start + command.length;
}
exports.end = end;
function getSelection(editor) {
    return {
        start: editor.document.offsetAt(editor.selection.start),
        end: editor.document.offsetAt(editor.selection.end),
        cursor: editor.document.offsetAt(editor.selection.active)
    };
}
exports.getSelection = getSelection;
function select(editor, pos) {
    const { pos1, pos2 } = toVscodePositions(editor, pos), sel = new vscode_1.Selection(pos1, pos2);
    editor.selection = sel;
    editor.revealRange(sel);
    return sel;
}
exports.select = select;
function copy(editor, pos) {
    const { pos1, pos2 } = toVscodePositions(editor, pos), range = new vscode_1.Range(pos1, pos2), text = editor.document.getText(range);
    clipboardy.writeSync(text);
}
exports.copy = copy;
function cut(editor, pos) {
    const command = toCommand(["delete", Math.min(pos[0], pos[1]), Math.abs(pos[1] - pos[0])]);
    copy(editor, pos);
    exports.edit(editor, [command]);
}
exports.cut = cut;
exports.handle = (editor, command) => edit => {
    let start = editor.document.positionAt(command.start);
    if (command.kind === 'insert')
        edit.insert(start, command.text);
    else {
        let end = editor.document.positionAt(command.start + command.length);
        edit.delete(new vscode_1.Selection(start, end));
    }
};
exports.edit = (editor, commands) => commands
    .reduce((prev, command) => prev.then((_) => editor.edit(exports.handle(editor, command), { undoStopAfter: false, undoStopBefore: false })), Promise.resolve(true));
function undoStop(editor) {
    let pos = editor.document.positionAt(0);
    editor.edit((edit) => edit.insert(pos, ""), { undoStopAfter: true, undoStopBefore: false });
}
exports.undoStop = undoStop;
//# sourceMappingURL=utils.js.map