"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const state = require("../state");
const util = require("../utilities");
const wsl_path_1 = require("wsl-path");
class DefinitionProvider {
    constructor() {
        this.state = state;
    }
    provideDefinition(document, position, token) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = util.getWordAtPosition(document, position);
            let client = util.getSession(util.getFileType(document));
            if (this.state.deref().get('connected')) {
                let info = yield client.info(util.getNamespace(document.getText()), text);
                if (info.file && info.file.length > 0) {
                    let pos = new vscode.Position(info.line - 1, info.column);
                    return new vscode.Location(vscode.Uri.parse(info.file), pos);
                }
            }
        });
    }
}
exports.DefinitionProvider = DefinitionProvider;
;
class WslDefinitionProvider extends DefinitionProvider {
    provideDefinition(document, position, token) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const location = yield _super("provideDefinition").call(this, document, position, token);
            if (!location)
                return;
            if (location.uri.scheme === 'jar') {
                const path = vscode.Uri.parse(location.uri.path).path;
                const windowsFilePath = yield wsl_path_1.wslToWindows(path);
                const windowsFileUri = vscode.Uri.file(windowsFilePath);
                return new vscode.Location(location.uri.with({ path: `file:${windowsFileUri.path}` }), location.range);
            }
            const windowsFilePath = yield wsl_path_1.wslToWindows(location.uri.path);
            return new vscode.Location(vscode.Uri.file(windowsFilePath), location.range);
        });
    }
}
exports.WslDefinitionProvider = WslDefinitionProvider;
//# sourceMappingURL=definition.js.map