"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const state = require("../state");
const util = require("../utilities");
class HoverProvider {
    constructor() {
        this.state = state;
    }
    formatDocString(nsname, arglist, doc) {
        let result = '';
        if (nsname.length > 0 && nsname !== 'undefined') {
            result += '**' + nsname + '**  ';
            result += '\n';
        }
        // Format the different signatures for the fn
        if (arglist & arglist.substring) {
            result += arglist.substring(0, arglist.length)
                .replace(/\]/g, ']_')
                .replace(/\[/g, '* _[');
            result += '\n\n';
        }
        // Format the actual docstring
        if (doc) {
            result += doc.replace(/\s\s+/g, ' ');
            result += '  ';
        }
        return result.length > 0 ? result : "";
    }
    provideHover(document, position, _) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = util.getWordAtPosition(document, position);
            if (this.state.deref().get('connected')) {
                let client = util.getSession(util.getFileType(document));
                let ns = util.getNamespace(document.getText());
                let res = yield client.info(ns, text);
                if (res.doc)
                    return new vscode.Hover(this.formatDocString(res.ns + "/" + res.name, res["arglists-str"] || [], res.doc));
                if (res.ns)
                    return new vscode.Hover(this.formatDocString(res.ns + "/" + res.name, res["arglists-str"] || [], "No documentation available"));
            }
            else
                return new vscode.Hover("Not connected to nREPL..");
        });
    }
}
exports.default = HoverProvider;
;
//# sourceMappingURL=hover.js.map