"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const paredit = require("./paredit/extension");
const fmt = require("./calva-fmt/ts/extension");
const state = require("./state");
const jackIn = require("./nrepl/jack-in");
const status_1 = require("./status");
const connector_1 = require("./connector");
const terminal_1 = require("./terminal");
const completion_1 = require("./providers/completion");
const content_1 = require("./providers/content");
const hover_1 = require("./providers/hover");
const definition_1 = require("./providers/definition");
const evaluate_1 = require("./repl/middleware/evaluate");
const lint_1 = require("./repl/middleware/lint");
const testRunner_1 = require("./repl/middleware/testRunner");
const annotations_1 = require("./providers/annotations");
const select_1 = require("./repl/middleware/select");
const evaluate_2 = require("./repl/middleware/evaluate");
const replWindow = require("./repl-window");
const greetings = require("./greet");
function onDidSave(document) {
    let { evaluate, lint, test } = state.config();
    if (document.languageId !== 'clojure') {
        return;
    }
    if (test) {
        if (test) {
            testRunner_1.default.runNamespaceTests(document);
        }
    }
    else if (evaluate) {
        evaluate_1.default.evaluateFile(document);
    }
    if (lint) {
        lint_1.default.lintDocument(document);
    }
}
function onDidOpen(document) {
    if (document.languageId !== 'clojure') {
        return;
    }
    if (state.config().lint) {
        lint_1.default.lintDocument(document);
    }
}
function activate(context) {
    let fmtExtension = vscode.extensions.getExtension('cospaia.calva-fmt'), pareEditExtension = vscode.extensions.getExtension('cospaia.paredit-revived');
    state.setExtensionContext(context);
    if (!fmtExtension) {
        fmt.activate(context);
    }
    else {
        vscode.window.showErrorMessage("Calva Format extension detected, which will break things. Please uninstall it before continuing using Calva.", ...["Got it. Will do!"]);
    }
    if (!pareEditExtension) {
        paredit.activate(context);
    }
    else {
        vscode.window.showErrorMessage("Calva Paredit extension detected, which can cause pronlems. Please uninstall it.", ...["Got it. Doing it!"]);
    }
    replWindow.activate(context);
    let chan = state.deref().get('outputChannel');
    chan.appendLine("Calva activated.");
    let { autoConnect, lint } = state.config();
    status_1.default.update();
    // COMMANDS
    context.subscriptions.push(vscode.commands.registerCommand('calva.jackIn', jackIn.calvaJackIn));
    context.subscriptions.push(vscode.commands.registerCommand('calva.connect', connector_1.default.connect));
    context.subscriptions.push(vscode.commands.registerCommand('calva.reconnect', connector_1.default.reconnect));
    context.subscriptions.push(vscode.commands.registerCommand('calva.toggleCLJCSession', connector_1.default.toggleCLJCSession));
    context.subscriptions.push(vscode.commands.registerCommand('calva.recreateCljsRepl', connector_1.default.recreateCljsRepl));
    context.subscriptions.push(vscode.commands.registerCommand('calva.selectCurrentForm', select_1.default.selectCurrentForm));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateFile', evaluate_1.default.evaluateFile));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateSelection', evaluate_1.default.evaluateSelection));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateCurrentTopLevelForm', evaluate_1.default.evaluateTopLevelForm));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateSelectionPrettyPrint', evaluate_1.default.evaluateSelectionPrettyPrint));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateCurrentTopLevelFormPrettyPrint', evaluate_1.default.evaluateCurrentTopLevelFormPrettyPrint));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evaluateSelectionReplace', evaluate_1.default.evaluateSelectionReplace));
    context.subscriptions.push(vscode.commands.registerCommand('calva.lintFile', lint_1.default.lintDocument));
    context.subscriptions.push(vscode.commands.registerCommand('calva.runNamespaceTests', testRunner_1.default.runNamespaceTestsCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.runAllTests', testRunner_1.default.runAllTestsCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.rerunTests', testRunner_1.default.rerunTestsCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.loadNamespace', terminal_1.default.loadNamespaceCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.setREPLNamespace', terminal_1.default.setREPLNamespaceCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evalCurrentFormInREPLTerminal', terminal_1.default.evalCurrentFormInREPLTerminalCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.evalCurrentTopLevelFormInREPLTerminal', terminal_1.default.evalCurrentTopLevelFormInREPLTerminalCommand));
    context.subscriptions.push(vscode.commands.registerCommand('calva.clearInlineResults', annotations_1.default.clearEvaluationDecorations));
    context.subscriptions.push(vscode.commands.registerCommand('calva.copyLastResults', evaluate_2.default.copyLastResultCommand));
    // PROVIDERS
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider(state.mode, new completion_1.default()));
    context.subscriptions.push(vscode.languages.registerHoverProvider(state.mode, new hover_1.default()));
    context.subscriptions.push(vscode.languages.registerDefinitionProvider(state.mode, new definition_1.default()));
    vscode.workspace.registerTextDocumentContentProvider('jar', new content_1.default());
    // //EVENTS
    context.subscriptions.push(vscode.workspace.onDidOpenTextDocument((document) => {
        onDidOpen(document);
    }));
    context.subscriptions.push(vscode.workspace.onDidSaveTextDocument((document) => {
        onDidSave(document);
    }));
    context.subscriptions.push(vscode.window.onDidChangeActiveTextEditor((editor) => {
        paredit.updatePareditEnabled();
        status_1.default.update();
        if (state.config().syncReplNamespaceToCurrentFile) {
            terminal_1.default.setREPLNamespace();
        }
    }));
    context.subscriptions.push(vscode.workspace.onDidChangeTextDocument(annotations_1.default.onDidChangeTextDocument));
    context.subscriptions.push(new vscode.Disposable(() => {
        connector_1.default.disconnect();
        chan.dispose();
    }));
    // context.subscriptions.push(vscode.workspace.onDidChangeConfiguration((event) => {
    //     console.log(event);
    // }));
    vscode.commands.executeCommand('setContext', 'calva:activated', true);
    greetings.activationGreetings(chan, lint);
    //Try to connect using an existing .nrepl-port file, searching the root-directory
    if (autoConnect) {
        chan.appendLine("Autoconnecting... (This can be disabled in Settings)");
        connector_1.default.autoConnect();
    }
    else {
        chan.appendLine("Autoconnect disabled in Settings.");
    }
    return {
        hasParedit: true,
        hasFormatter: true
    };
}
exports.activate = activate;
function deactivate() {
    paredit.deactivate();
    jackIn.killAllProcesses();
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map