"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const state = require("../state");
const util = require("../utilities");
class CalvaCompletionItemProvider {
    constructor() {
        this.state = state;
        this.mappings = {
            'nil': vscode_1.CompletionItemKind.Value,
            'macro': vscode_1.CompletionItemKind.Value,
            'class': vscode_1.CompletionItemKind.Class,
            'keyword': vscode_1.CompletionItemKind.Keyword,
            'namespace': vscode_1.CompletionItemKind.Module,
            'function': vscode_1.CompletionItemKind.Function,
            'special-form': vscode_1.CompletionItemKind.Keyword,
            'var': vscode_1.CompletionItemKind.Variable,
            'method': vscode_1.CompletionItemKind.Method
        };
    }
    provideCompletionItems(document, position, token, context) {
        return __awaiter(this, void 0, void 0, function* () {
            let text = util.getWordAtPosition(document, position);
            if (this.state.deref().get("connected")) {
                let client = util.getSession(util.getFileType(document));
                let res = yield client.complete(util.getNamespace(document.getText()), text);
                let results = res.completions || [];
                return new vscode_1.CompletionList(results.map(item => ({
                    label: item.candidate,
                    kind: this.mappings[item.type] || vscode_1.CompletionItemKind.Text,
                    insertText: item[0] === '.' ? item.slice(1) : item
                })), true);
            }
            return [];
        });
    }
    resolveCompletionItem(item, token) {
        return __awaiter(this, void 0, void 0, function* () {
            let client = util.getSession(util.getFileType(vscode_1.window.activeTextEditor.document));
            if (this.state.deref().get("connected")) {
                let result = yield client.info(item.insertText["ns"], item.label);
                if (result.doc)
                    item.documentation = result.doc;
            }
            return item;
        });
    }
}
exports.default = CalvaCompletionItemProvider;
;
//# sourceMappingURL=completion.js.map