"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const clipboardy = require("clipboardy");
const state = require("../../state");
const annotations_1 = require("../../providers/annotations");
const select_1 = require("./select");
const util = require("../../utilities");
/// FIXME: We need to add pprint options back in.
function evaluateSelection(document = {}, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        let current = state.deref(), chan = current.get('outputChannel'), doc = util.getDocument(document), pprint = options["pprint"] || false, replace = options["replace"] || false, topLevel = options["topLevel"] || false;
        if (current.get('connected')) {
            let client = util.getSession(util.getFileType(doc));
            let editor = vscode.window.activeTextEditor, selection = editor.selection, codeSelection = null, code = "";
            if (selection.isEmpty) {
                codeSelection = select_1.default.getFormSelection(doc, selection.active, topLevel);
                code = doc.getText(codeSelection);
            }
            else {
                codeSelection = selection;
                code = doc.getText(selection);
            }
            if (code.length > 0) {
                annotations_1.default.decorateSelection(codeSelection, editor, annotations_1.default.AnnotationStatus.PENDING);
                let c = codeSelection.start.character;
                let err = [], out = [];
                let res = yield client.eval("(in-ns '" + util.getNamespace(doc.getText()) + ")").value;
                try {
                    let context = client.eval(code, { stdout: m => out.push(m), stderr: m => err.push(m), pprint: !!pprint });
                    let value = yield context.value;
                    value = context.pprintOut || value;
                    if (replace) {
                        const indent = ' '.repeat(c), edit = vscode.TextEdit.replace(codeSelection, value.replace(/\n/gm, "\n" + indent)), wsEdit = new vscode.WorkspaceEdit();
                        wsEdit.set(editor.document.uri, [edit]);
                        vscode.workspace.applyEdit(wsEdit);
                        chan.appendLine("Replaced inline.");
                    }
                    else {
                        annotations_1.default.decorateSelection(codeSelection, editor, annotations_1.default.AnnotationStatus.SUCCESS);
                        if (!pprint)
                            annotations_1.default.decorateResults(' => ' + value.replace(/\n/gm, " ") + " ", false, codeSelection, editor);
                    }
                    if (out.length > 0) {
                        chan.append("out: ");
                        chan.append(out.join("\n"));
                    }
                    chan.append('=> ');
                    if (pprint) {
                        chan.appendLine('');
                        chan.show(true);
                        chan.appendLine(value);
                    }
                    else
                        chan.appendLine(value);
                    if (err.length > 0) {
                        chan.append("Error: ");
                        chan.append(err.join("\n"));
                    }
                }
                catch (e) {
                    if (!err.length) { // venantius/ultra outputs errors on stdout, it seems.
                        err = out;
                        if (err.length > 0) {
                            chan.append("Error: ");
                            chan.append(err.join("\n"));
                        }
                    }
                    annotations_1.default.decorateSelection(codeSelection, editor, annotations_1.default.AnnotationStatus.ERROR);
                    if (!pprint) {
                        const annotation = err.join();
                        annotations_1.default.decorateResults(' => ' + annotation.replace(/\n/gm, " ") + " ", true, codeSelection, editor);
                    }
                }
            }
        }
        else
            vscode.window.showErrorMessage("Not connected to a REPL");
    });
}
function evaluateSelectionReplace(document = {}, options = {}) {
    evaluateSelection(document, Object.assign({}, options, { replace: true, pprint: true }));
}
function evaluateSelectionPrettyPrint(document = {}, options = {}) {
    evaluateSelection(document, Object.assign({}, options, { pprint: true }));
}
function evaluateCurrentTopLevelFormPrettyPrint(document = {}, options = {}) {
    evaluateSelection(document, Object.assign({}, options, { pprint: true, topLevel: true }));
}
function evaluateTopLevelForm(document = {}, options = {}) {
    evaluateSelection(document, Object.assign({}, options, { topLevel: true }));
}
function evaluateFile(document = {}, callback = () => { }) {
    return __awaiter(this, void 0, void 0, function* () {
        let current = state.deref(), doc = util.getDocument(document), fileName = util.getFileName(doc), fileType = util.getFileType(doc), client = util.getSession(util.getFileType(doc)), chan = current.get('outputChannel');
        if (doc.languageId == "clojure" && fileType != "edn" && current.get('connected')) {
            chan.appendLine("Evaluating file: " + fileName);
            let value = yield client.loadFile(doc.getText(), { fileName: fileName, filePath: doc.fileName }).value;
            if (value !== null)
                chan.appendLine("=> " + value);
            else
                chan.appendLine("No results from file evaluation.");
        }
        callback();
    });
}
function copyLastResultCommand() {
    return __awaiter(this, void 0, void 0, function* () {
        let chan = state.deref().get('outputChannel');
        let client = util.getSession(util.getFileType(util.getDocument({})));
        let value = yield client.eval("*1").value;
        if (value !== null)
            clipboardy.writeSync(value);
        else
            chan.appendLine("Nothing to copy");
    });
}
exports.default = {
    evaluateFile,
    evaluateSelection,
    evaluateTopLevelForm,
    evaluateSelectionPrettyPrint,
    evaluateCurrentTopLevelFormPrettyPrint,
    evaluateSelectionReplace,
    copyLastResultCommand
};
//# sourceMappingURL=evaluate.js.map